import { A as Annotation, M as MapStyleSpecification, S as StateStore, a as MapDataSource, b as MapLayer, F as FlowContextOptions, c as FlowContext } from './shared/flow-rdf-core.08156446.js';
export { B as BackgroundLayerStyleBuilder, C as CircleLayerStyleBuilder, D as DEFAULT_CONTEXT_ID, _ as FeatureStreamOptions, s as FillExtrusionLayerStyleBuilder, q as FillLayerStyleBuilder, d as FlowCoreError, h as FlowCoreExpressionCustomFunction, i as FlowCoreExpressionCustomFunctionSignature, j as FlowCoreExpressionCustomFunctions, e as FlowCoreExpressionError, a0 as FlowCoreMap, o as FlowEngineEndpoint, f as FlowMapError, H as HeatmapLayerStyleBuilder, u as HillshadeLayerStyleBuilder, L as LineLayerStyleBuilder, T as MapAnimationOptions, U as MapCenterOptions, K as MapDataSourceOptions, N as MapDataSourceSpecification, J as MapDataSourceType, a3 as MapEvent, a2 as MapEventType, a1 as MapEventTypes, V as MapFeature, W as MapFeatureIdentifier, Z as MapFeatureStream, O as MapGeojsonData, E as MapLayerExpression, z as MapLayerOptions, I as MapLayerPropertyMetadataIndex, G as MapLayerSpecification, w as MapLayerStyleBuilder, p as MapLayerType, a6 as MapLoadingEvent, a4 as MapMouseEvent, $ as MapOptions, X as MapPaddingOptions, aa as MapPopup, a9 as MapPopupOptions, a8 as MapPopupTemplateFn, g as MapRegistry, a5 as MapSourceEvent, Y as MapSourceIdentifier, a7 as MapStyleChangeEvent, P as ParserExpression, k as ParserExpressionWithIndex, l as ParserFunctionExpression, m as ParserIndex, n as ParserProgram, aq as Path, ar as PathValue, ab as Query, ag as QueryAttribute, al as QueryDefinition, an as QueryDefinitionFn, ai as QueryExtraParams, am as QueryOptions, aj as QueryOrderClause, Q as QueryRegistry, af as QueryResultFormat, ao as QueryResultTransformFn, ak as QuerySampleOptions, ah as QueryWhereClauses, t as RasterLayerStyleBuilder, R as ReactiveExpression, au as SerializationFormat, at as StoreAnnotations, as as StoreConstructor, x as StyleExpression, r as SymbolLayerStyleBuilder, y as expr, v as layerStyle, ac as query, ad as queryMany, ae as queryOne, ap as traceReaction } from './shared/flow-rdf-core.08156446.js';
export { IDisposer, IObservableStream, ISubscription } from 'mobx-utils';
export { makeAutoObservable, makeObservable } from 'mobx';
import 'eventemitter3';
import 'lit-html';
import '@emuanalytics/flow-engine-client';
import '@emuanalytics/maplibre-gl';

type SyncAnnotationArg = string | string[] | {
    scope?: string | string[];
    name?: string;
};
interface SyncAnnotation<T = any> extends Annotation {
    serialize(val: T, scope?: string): string;
    deserialize(val: string, target?: T): T;
    type: string;
    arg?: SyncAnnotationArg;
}
declare const serialize: PropertyDecorator & SyncAnnotation<any> & {
    raw: PropertyDecorator & SyncAnnotation<any>;
    string: PropertyDecorator & SyncAnnotation<string>;
    number: PropertyDecorator & SyncAnnotation<number>;
    json: PropertyDecorator & SyncAnnotation<Record<string, boolean>>;
    state: PropertyDecorator & SyncAnnotation<any>;
    for: (arg: SyncAnnotationArg | undefined) => PropertyDecorator & SyncAnnotation<any>;
};
interface ReactiveAnnotation extends Annotation {
    mobxAnnotation: any;
}
declare const observable: PropertyDecorator & ReactiveAnnotation & {
    deep: PropertyDecorator & ReactiveAnnotation;
    ref: PropertyDecorator & ReactiveAnnotation;
    struct: PropertyDecorator & ReactiveAnnotation;
    shallow: PropertyDecorator & ReactiveAnnotation;
};
declare const computed: PropertyDecorator & ReactiveAnnotation & {
    struct: PropertyDecorator & ReactiveAnnotation;
};
declare const action: PropertyDecorator & ReactiveAnnotation;

interface MapStyleOptions {
    id: string;
}
declare class MapStyle {
    private static _defaultBaseStyle;
    private _state;
    private _baseStyle;
    private _sources;
    private _layers;
    private _layerOrder;
    get styleSpecification(): MapStyleSpecification;
    get formattedName(): string;
    constructor();
    bindState(state: StateStore): this;
    setBaseStyle(style?: MapStyleSpecification): this;
    addSource(source: MapDataSource): this;
    removeSource(id: string, removeLayers?: boolean): this;
    getSource(id: string): MapDataSource;
    getSources(filter?: (source: MapDataSource) => boolean): MapDataSource[];
    addLayer(layer: MapLayer): this;
    removeLayer(id: string): this;
    getLayer(id: string): MapLayer;
    getLayers(filter?: (layer: MapLayer) => boolean): MapLayer[];
    private layerInsertionPoint;
}

declare class ContextRegistry {
    private static _defaultOptions;
    private _contexts;
    /**
     * Initialize a new Flo.w context.
     *
     * @param options Context options.
     */
    initializeContext<TState extends StateStore = StateStore>(options: FlowContextOptions<TState>): FlowContext<TState>;
    uninitializeContext(contextId?: string): void;
    getContext<TState extends StateStore = StateStore>(id?: string): Promise<FlowContext<TState>>;
}

declare class FlowRdfCoreGlobals {
    /**
     * FlowRdfCoreGlobals version.
     * Compatiblity with other versions loaded in memory as long as this version matches.
     * It indicates that the global state still stores similar information
     *
     * N.B: this version is unrelated to the package version.
     */
    version: number;
    globalContextRegistry: ContextRegistry;
}
declare function isolateGlobalState(): void;
declare function getGlobalState(): FlowRdfCoreGlobals;

declare const initializeContext: <TState extends StateStore = StateStore>(options: FlowContextOptions<TState>) => FlowContext<TState>;
declare const uninitializeContext: (contextId?: string) => void;
declare const getContext: <TState extends StateStore = StateStore>(id?: string) => Promise<FlowContext<TState>>;

export { ContextRegistry, FlowContext, FlowContextOptions, FlowRdfCoreGlobals, MapDataSource, MapLayer, MapStyle, type MapStyleOptions, MapStyleSpecification, StateStore, action, computed, getContext, getGlobalState, initializeContext, isolateGlobalState, observable, serialize, uninitializeContext };
