import Debug from 'debug';
import { deferred } from '../utils/deferred.mjs';
import { FlowContext, DEFAULT_CONTEXT_ID } from './flow-context.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const debug = Debug("flow:context-registry");
const _ContextRegistry = class _ContextRegistry {
  constructor() {
    __publicField(this, "_contexts", /* @__PURE__ */ new Map());
  }
  /**
   * Initialize a new Flo.w context.
   *
   * @param options Context options.
   */
  initializeContext(options) {
    options = { ..._ContextRegistry._defaultOptions, ...options };
    debug(`Initializing context '${options.id}'`, options);
    if (!this._contexts.get(options.id)) {
      this._contexts.set(options.id, {
        initialized: false,
        deferred: deferred()
      });
    }
    const registryEntry = this._contexts.get(options.id);
    if (registryEntry.initialized) {
      console.warn(`Context '${options.id}' has already been initialized. Skipping re-initialization.`);
    }
    const context = new FlowContext(options);
    registryEntry.initialized = true;
    registryEntry.deferred.resolve(context);
    return context;
  }
  uninitializeContext(contextId = DEFAULT_CONTEXT_ID) {
    if (!this._contexts.has(contextId)) {
      console.warn(`Context '${contextId}' has not been initialized. Skipping uninitialization.`);
    }
    debug(`Uninitializing context '${contextId}'`);
    this._contexts.delete(contextId);
  }
  getContext(id = DEFAULT_CONTEXT_ID) {
    if (!this._contexts.get(id)) {
      this._contexts.set(id, {
        initialized: false,
        deferred: deferred()
      });
    }
    const deferredContext = this._contexts.get(id);
    return deferredContext?.deferred;
  }
};
__publicField(_ContextRegistry, "_defaultOptions", {
  id: DEFAULT_CONTEXT_ID,
  host: "https://flow.emu-analytics.net"
});
let ContextRegistry = _ContextRegistry;

export { ContextRegistry };
