import { Client } from '@emuanalytics/flow-engine-client';
import { Debugger, installGlobalDebugger } from './debugger.mjs';
import { MapRegistry } from './map-registry.mjs';
import { QueryRegistry } from './query-registry.mjs';
import { ReactiveExpression } from '../expressions/reactive-expression.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const DEFAULT_FLOW_HOST = "https://flow.emu-analytics.net";
const DEFAULT_CONTEXT_ID = "default";
const _FlowContext = class _FlowContext {
  /**
   * Constructs a new `FlowCoreContext`.
   *
   * Use [[ContextRegistry.initializeContext]] to create an instance of `FlowCoreContext`.
   *
   * @param options - The context options.
   */
  constructor(options) {
    __publicField(this, "_options");
    __publicField(this, "flowClient");
    __publicField(this, "state");
    __publicField(this, "maps", new MapRegistry());
    __publicField(this, "queries", new QueryRegistry());
    /** The Flo.w debugger. Available as `window.FDBG` in the browser's web developer console. */
    __publicField(this, "debugger");
    this._options = {
      ..._FlowContext._defaultOptions,
      ...options
    };
    if (!options.state.initialized) {
      options.state.initialize();
    }
    this.flowClient = new Client(this._options.host, this._options.apiKey, {
      withCredentials: true,
      transports: ["polling", "websocket"]
    });
    this.state = options.state;
    if (options.enableDebug) {
      this.debugger = new Debugger(this);
      installGlobalDebugger(this.debugger);
    }
  }
  get id() {
    return this._options.id;
  }
  evaluateExpression(expr) {
    const expression = new ReactiveExpression(expr);
    expression.bindState(this.state);
    return expression.value;
  }
};
__publicField(_FlowContext, "_defaultOptions", {
  id: DEFAULT_CONTEXT_ID,
  host: DEFAULT_FLOW_HOST,
  enableDebug: false
});
let FlowContext = _FlowContext;

export { DEFAULT_CONTEXT_ID, FlowContext };
