import Debug from 'debug';
import { deferred } from '../utils/deferred.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const debug = Debug("flow:map-registry");
class MapRegistry {
  constructor() {
    __publicField(this, "_maps", /* @__PURE__ */ new Map());
  }
  /**
   * Register a Flo.w map.
   *
   * @param map Map to register.
   */
  registerMap(map) {
    debug(`Registering map '${map.id}'`);
    if (!this._maps.get(map.id)) {
      this._maps.set(map.id, deferred());
    }
    const deferredMap = this._maps.get(map.id);
    deferredMap.resolve(map);
    return map;
  }
  /**
   * Un-register a Flo.w map.
   *
   * ```mermaid
   * graph TB
   *   mermaid.js --> TypeDoc;
   * ```
   *
   * @param map - the map to unregister.
   */
  unregisterMap(map) {
    debug(`Unregistering map '${map.id}'`);
    this._maps.delete(map.id);
  }
  /**
   * Retrieve the specified map.
   *
   * This method returns a promise that resolves when the map has been initialized.
   *
   * @param id Map ID.
   */
  getMap(id) {
    if (!this._maps.get(id)) {
      this._maps.set(id, deferred());
    }
    const deferredMap = this._maps.get(id);
    return deferredMap;
  }
}

export { MapRegistry };
