import Debug from 'debug';
import { deferred } from '../utils/deferred.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const debug = Debug("flow:query-registry");
class QueryRegistry {
  constructor() {
    __publicField(this, "_queries", /* @__PURE__ */ new Map());
  }
  /**
   * Register a Flo.w reactive query.
   *
   * @param query Query to register.
   */
  registerQuery(query) {
    debug(`Registering query '${query.id}'`);
    if (!this._queries.get(query.id)) {
      this._queries.set(query.id, deferred());
    }
    const deferredQuery = this._queries.get(query.id);
    deferredQuery.resolve(query);
  }
  /**
   * Un-register a Flo.w reactive query.
   *
   * @param query Query to unregister.
   */
  unregisterQuery(query) {
    debug(`Unregistering map '${query.id}'`);
    this._queries.delete(query.id);
  }
  /**
   * Retrieve the specified query.
   *
   * This method returns a promise that resolves when the query has been initialized.
   *
   * @param id Map ID.
   */
  getQuery(id) {
    if (!this._queries.get(id)) {
      this._queries.set(id, deferred());
    }
    const deferredQuery = this._queries.get(id);
    return deferredQuery;
  }
}

export { QueryRegistry };
