import { isNil } from '../../utils/type-checks.mjs';

function stateCustomFunction(expr, parentExpression) {
  if (!parentExpression.store) {
    console.warn(
      `The reactive expression '${parentExpression.expression}' references state properties but is not bound to a state store. The expression cannot be evaluated - returning null.`
    );
    return null;
  }
  const path = expr.slice(1).join(".");
  const val = parentExpression.store.get(path);
  if (isNil(val)) {
    return null;
  } else if (typeof val === "object") {
    return ["literal", val];
  } else {
    return val;
  }
}
const customStateFn = {
  name: "state",
  fn: stateCustomFunction
};

export { customStateFn as default };
