import { isString, isNumber, isBoolean, isArray } from '../utils/type-checks.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class MapLayerStyleBuilder {
  constructor() {
    __publicField(this, "_styles", /* @__PURE__ */ new Map());
  }
  static registerStyleBuilder(layerType, builder) {
    this._styleBuilders.set(layerType, builder);
  }
  static createStyleBuilder(layerType) {
    const builder = this._styleBuilders.get(layerType);
    if (!builder) {
      throw new Error(`MapLayerStyleBuilder: Unknown layer type '${layerType}'`);
    }
    return new builder();
  }
  /**
   * Whether this layer is displayed.
   *
   * @param value - visible value (default `true`)
   */
  visibility(value) {
    this.addStyle("visibility", value);
    return this;
  }
  /**
   * Create a style specification string.
   *
   * Call this method after setting all style properties
   * on the style builder.
   *
   * @returns Layer style specification string
   */
  build() {
    const styles = [];
    for (const [prop, value] of this._styles.entries()) {
      styles.push(`${prop}: ${value};`);
    }
    return styles.join("\n");
  }
  /**
   * Format and add style property to the style builder.
   */
  addStyle(property, value) {
    this._styles.set(property, this.formatValue(value));
  }
  formatValue(value) {
    if (isStyleExpression(value)) {
      return value.toString();
    } else if (isString(value)) {
      return `'${value.toString()}'`;
    } else if (isNumber(value)) {
      return value.toString();
    } else if (isBoolean(value)) {
      return (!!value).toString();
    } else if (isArray(value)) {
      return `[${value.map((v) => this.formatValue(v)).join(", ")}]`;
    } else {
      return value.toString();
    }
  }
}
__publicField(MapLayerStyleBuilder, "_styleBuilders", /* @__PURE__ */ new Map());
class StyleExpression {
  constructor(_expr) {
    this._expr = _expr;
    __publicField(this, "__styleExpression", "styleexpression");
  }
  toString() {
    return this._expr;
  }
}
function isStyleExpression(val) {
  return val && val.__styleExpression;
}
function expr(expr2) {
  return new StyleExpression(expr2);
}

export { MapLayerStyleBuilder, StyleExpression, expr };
