import { MapDataSource } from '../../../map-style/map-data-source.mjs';
import { MapLayer } from '../../../map-style/map-layer.mjs';

function addAnnotationLayers(map) {
  map.transaction(() => {
    const annotationSource = new MapDataSource({
      id: "map-annotations",
      type: "geojson",
      src: { type: "FeatureCollection", features: [] },
      featureId: "id"
    });
    map.addSource(annotationSource);
    const polygonFillLayer = new MapLayer({
      id: "map-annotations-polygon-fill",
      type: "fill",
      source: annotationSource.id,
      filter: `geometry-type() == 'Polygon' && coalesce(@visible, true)`,
      style: `
      fill-color: @fillColor;
      `
    });
    map.addLayer(polygonFillLayer);
    const polygonStrokeLayer = new MapLayer({
      id: "map-annotations-polygon-stroke",
      type: "line",
      source: annotationSource.id,
      filter: `geometry-type() == 'Polygon' && coalesce(@visible, true)`,
      style: `
    line-color: @strokeColor;
    line-width: @strokeWidth;
    `
    });
    map.addLayer(polygonStrokeLayer);
    const lineStrokeLayer = new MapLayer({
      id: "map-annotations-line-stroke",
      type: "line",
      source: annotationSource.id,
      filter: `geometry-type() == 'LineString' && coalesce(@visible, true)`,
      style: `
    line-color: @strokeColor;
    line-width: @strokeWidth;
    `
    });
    map.addLayer(lineStrokeLayer);
    const pointLayer = new MapLayer({
      id: "map-annotations-point",
      type: "circle",
      source: annotationSource.id,
      filter: `geometry-type() == 'Point' && coalesce(@visible, true)`,
      style: `
        circle-radius: @radius;
        circle-color: @fillColor;
        circle-stroke-color: @strokeColor;
        circle-stroke-width: @strokeWidth;
      `
    });
    map.addLayer(pointLayer);
    const polygonLabelsLayer = new MapLayer({
      id: "map-annotations-polygon-labels",
      type: "symbol",
      source: annotationSource.id,
      filter: `geometry-type() == 'Polygon' && to-boolean(@labelText) && coalesce(@visible, true)`,
      interactive: false,
      style: `
        symbol-placement: 'point';
        text-field: @labelText;
        text-color: @labelColor;
        text-size: @labelSize;
        text-anchor: @polygonLabelAnchor;
        /* text-offset: @labelOffset; */
        text-justify: @labelJustify;
        text-max-width: @labelMaxWidth;
        text-allow-overlap: true;
      `
    });
    map.addLayer(polygonLabelsLayer);
    const pointLabelsLayer = new MapLayer({
      id: "map-annotations-point-labels",
      type: "symbol",
      source: annotationSource.id,
      filter: `geometry-type() == 'Point' && to-boolean(@labelText) && coalesce(@visible, true)`,
      interactive: false,
      style: `
        symbol-placement: 'point';
        text-field: @labelText;
        text-color: @labelColor;
        text-size: @labelSize;
        text-anchor: @labelAnchor;
        text-offset: @labelOffset;
        /* text-radial-offset: (@radius + 8) / coalesce(@labelSize, {{annotationsStyleDefaults.labelSize}}); */
        text-justify: @labelJustify;
        text-max-width: @labelMaxWidth;
        text-allow-overlap: true;
      `
    });
    map.addLayer(pointLabelsLayer);
    const lineLabelsLayer = new MapLayer({
      id: "map-annotations-line-labels",
      type: "symbol",
      source: annotationSource.id,
      filter: `geometry-type() == 'LineString' && to-boolean(@labelText) && coalesce(@visible, true)`,
      interactive: false,
      style: `
        symbol-placement: 'line-center';
        text-field: @labelText;
        text-color: @labelColor;
        text-anchor: @labelAnchor;
        text-size: @labelSize;
        /* text-offset: @labelOffset; */
        text-justify: @labelJustify;
        text-max-width: @labelMaxWidth;
        text-allow-overlap: true;
        text-ignore-placement: true;
    `
    });
    map.addLayer(lineLabelsLayer);
    const polygonSelectionLayer = new MapLayer({
      id: "map-annotations-polygon-selection",
      type: "line",
      source: annotationSource.id,
      filter: `geometry-type() == 'Polygon' && coalesce(@visible, true)`,
      interactive: false,
      style: `
        line-color: @@selected ? @selectionStrokeColor : @@hover ? @hoverStrokeColor : transparent;
        line-width: @selectionWidth;
        line-opacity: @selectionStrokeOpacity;
      `
    });
    map.addLayer(polygonSelectionLayer);
    const pointSelectionLayer = new MapLayer({
      id: "map-annotations-point-selection",
      type: "circle",
      source: annotationSource.id,
      filter: `geometry-type() == 'Point' && coalesce(@visible, true)`,
      interactive: false,
      style: `
        circle-radius: @radius + 1;
        circle-color: transparent;
        circle-stroke-color: @@selected ? @selectionStrokeColor : @@hover ? @hoverStrokeColor : transparent;
        circle-stroke-width: @selectionWidth;
        circle-stroke-opacity: @selectionStrokeOpacity;
      `
    });
    map.addLayer(pointSelectionLayer);
    const lineSelectionLayer = new MapLayer({
      id: "map-annotations-line-selection",
      type: "line",
      source: annotationSource.id,
      filter: `geometry-type() == 'LineString' && coalesce(@visible, true)`,
      interactive: false,
      style: `
        line-color: @@selected ? @selectionStrokeColor : @@hover ? @hoverStrokeColor : transparent;
        line-opacity: @selectionStrokeOpacity;
        line-width: @selectionWidth/2;
        line-gap-width: @strokeWidth;
      `
    });
    map.addLayer(lineSelectionLayer);
  });
  return map.getSource("map-annotations");
}

export { addAnnotationLayers };
