import MapboxDraw from '@emuanalytics/mapbox-gl-draw';
import turfCircle from '@turf/circle';
import turfDistance from '@turf/distance';
import { point } from '@turf/helpers';
import createSupplementaryPointsForCircle from '../utils/create_supplementary_points_circle.mjs';

const DirectModeOverride = MapboxDraw.modes.direct_select;
const moveFeatures = MapboxDraw.lib.moveFeatures;
const constrainFeatureMovement = MapboxDraw.lib.constrainFeatureMovement;
const createSupplementaryPoints = MapboxDraw.lib.createSupplementaryPoints;
const Constants = MapboxDraw.constants;
DirectModeOverride.dragFeature = function(state, e, delta) {
  moveFeatures(this.getSelected(), delta);
  this.getSelected().filter((feature) => feature.properties.type === "Circle").map((circle) => circle.properties.circleCenter).forEach((center) => {
    center[0] += delta.lng;
    center[1] += delta.lat;
  });
  state.dragMoveLocation = e.lngLat;
};
DirectModeOverride.dragVertex = function(state, e, delta) {
  if (state.feature.properties.type === "Circle") {
    const center = state.feature.properties.circleCenter;
    const movedVertex = [e.lngLat.lng, e.lngLat.lat];
    const radius = turfDistance(point(center), point(movedVertex), { units: "kilometers" });
    const circleFeature = turfCircle(center, radius);
    state.feature.incomingCoords(circleFeature.geometry.coordinates);
    state.feature.properties.circleRadius = radius;
  } else {
    const selectedCoords = state.selectedCoordPaths.map((coord_path) => state.feature.getCoordinate(coord_path));
    const selectedCoordPoints = selectedCoords.map((coords) => ({
      type: Constants.geojsonTypes.FEATURE,
      properties: {},
      geometry: {
        type: Constants.geojsonTypes.POINT,
        coordinates: coords
      }
    }));
    const constrainedDelta = constrainFeatureMovement(selectedCoordPoints, delta);
    for (let i = 0; i < selectedCoords.length; i++) {
      const coord = selectedCoords[i];
      state.feature.updateCoordinate(
        state.selectedCoordPaths[i],
        coord[0] + constrainedDelta.lng,
        coord[1] + constrainedDelta.lat
      );
    }
  }
};
DirectModeOverride.toDisplayFeatures = function(state, geojson, push) {
  if (state.featureId === geojson.properties.id) {
    geojson.properties.active = Constants.activeStates.ACTIVE;
    push(geojson);
    const supplementaryPoints = geojson.properties.user_type === "Circle" ? createSupplementaryPointsForCircle(geojson) : createSupplementaryPoints(geojson, {
      map: this.map,
      midpoints: true,
      selectedPaths: state.selectedCoordPaths
    });
    supplementaryPoints.forEach(push);
  } else {
    geojson.properties.active = Constants.activeStates.INACTIVE;
    push(geojson);
  }
  this.fireActionable(state);
};
const DirectMode = DirectModeOverride;

export { DirectMode as default };
