import MapboxDraw from '@emuanalytics/mapbox-gl-draw';
import turfCircle from '@turf/circle';
import turfDistance from '@turf/distance';
import { point } from '@turf/helpers';
import dragPan from '../utils/drag_pan.mjs';

const DragCircleMode = { ...MapboxDraw.modes.draw_polygon };
const Constants = MapboxDraw.constants;
DragCircleMode.onSetup = function(opts) {
  const polygon = this.newFeature({
    type: Constants.geojsonTypes.FEATURE,
    properties: {
      type: "Circle",
      circleCenter: []
    },
    geometry: {
      type: Constants.geojsonTypes.POLYGON,
      coordinates: []
    }
  });
  this.addFeature(polygon);
  this.clearSelectedFeatures();
  dragPan.disable(this);
  this.updateUIClasses({ mouse: Constants.cursors.ADD });
  this.activateUIButton(Constants.types.POLYGON);
  this.setActionableState({
    trash: true
  });
  return {
    polygon,
    currentVertexPosition: 0
  };
};
DragCircleMode.onMouseDown = DragCircleMode.onTouchStart = function(state, e) {
  const currentCenter = state.polygon.properties.circleCenter;
  if (currentCenter.length === 0) {
    state.polygon.properties.circleCenter = [e.lngLat.lng, e.lngLat.lat];
  }
};
DragCircleMode.onDrag = DragCircleMode.onMouseMove = function(state, e) {
  const center = state.polygon.properties.circleCenter;
  if (center.length > 0) {
    const distanceInKm = turfDistance(point(center), point([e.lngLat.lng, e.lngLat.lat]), {
      units: "kilometers"
    });
    const circleFeature = turfCircle(center, distanceInKm);
    state.polygon.incomingCoords(circleFeature.geometry.coordinates);
    state.polygon.properties.circleRadius = distanceInKm;
  }
};
DragCircleMode.onMouseUp = DragCircleMode.onTouchEnd = function(state) {
  dragPan.enable(this);
  return this.changeMode(Constants.modes.SIMPLE_SELECT, { featureIds: [state.polygon.id] });
};
DragCircleMode.onClick = DragCircleMode.onTap = function(state) {
  state.polygon.properties.circleCenter = [];
};
DragCircleMode.toDisplayFeatures = function(state, geojson, display) {
  const isActivePolygon = geojson.properties.id === state.polygon.id;
  geojson.properties.active = isActivePolygon ? Constants.activeStates.ACTIVE : Constants.activeStates.INACTIVE;
  return display(geojson);
};
const DragCircleMode$1 = DragCircleMode;

export { DragCircleMode$1 as default };
