import { isDefined, isObject } from '../../utils/type-checks.mjs';
import { isFeatureWithId } from '../types.mjs';
import { MapPluginBase } from './map-plugin-base.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class HoverEffectPlugin extends MapPluginBase {
  constructor(pluginManager) {
    super(pluginManager);
    __publicField(this, "hoveredFeature", null);
    __publicField(this, "name", "HoverEffectPlugin");
  }
  handleMouseMove(event) {
    const hoveredFeature = event.features[0];
    if (hoveredFeature) {
      this.map.maplibreMap._container.classList.add("feature-hovered");
    } else {
      this.map.maplibreMap._container.classList.remove("feature-hovered");
    }
    if (!this.compareFeatures(hoveredFeature, this.hoveredFeature)) {
      this.clearHover();
      if (isFeatureWithId(hoveredFeature) && isDefined(this.map.getSource(hoveredFeature.source)?.featureId)) {
        this.map.setFeatureState(hoveredFeature, { hover: true }, false);
        this.hoveredFeature = hoveredFeature;
      }
    }
  }
  handleMouseOut(event) {
    this.clearHover();
  }
  handleSetExclusivePlugin(exclusivePlugins) {
    if (exclusivePlugins.length) {
      this.clearHover();
      this.map.maplibreMap._container.classList.remove("feature-hovered");
    }
  }
  compareFeatures(f1, f2) {
    if (f1 === f2) {
      return true;
    }
    if (isObject(f1) && isObject(f2)) {
      return f1.id == f2.id && f1.source === f2.source && f1.sourceLayer == f2.sourceLayer;
    }
    return false;
  }
  clearHover() {
    if (this.hoveredFeature && this.map.getSource(this.hoveredFeature.source)) {
      this.map.removeFeatureState(this.hoveredFeature, "hover", false);
      this.hoveredFeature = null;
    }
  }
}

export { HoverEffectPlugin };
