import { ensureArray } from '../../utils/ensure-array.mjs';
import { isArray } from '../../utils/type-checks.mjs';
import { MapPopup } from '../support/map-popup.mjs';
import { MapPluginBase } from './map-plugin-base.mjs';
import { FlowCoreError } from '../../errors/flow-error.mjs';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class PopupPlugin extends MapPluginBase {
  constructor(pluginManager) {
    super(pluginManager);
    __publicField(this, "_popupTemplates", /* @__PURE__ */ new Map());
    __publicField(this, "_popups", /* @__PURE__ */ new Set());
    __publicField(this, "name", "PopupPlugin");
  }
  registerPopup(id, options) {
    this._popupTemplates.set(id, options);
  }
  openPopup(id, eventOrLngLat, feature) {
    const popupOptions = this._popupTemplates.get(id);
    if (!popupOptions) {
      throw new FlowCoreError(`MapPopup: no popup with id '${id}'`);
    }
    const popup = new MapPopup(this.map, popupOptions);
    this._popups.add(popup);
    popup.once("closed", () => {
      this._popups.delete(popup);
    });
    if (isArray(eventOrLngLat)) {
      popup.open(eventOrLngLat, feature);
    } else {
      popup.open(eventOrLngLat);
    }
    return popup;
  }
  closeAllPopups() {
    this._popups.forEach((popup) => popup.close());
    this._popups.clear();
  }
  closePopupsForSources(sources) {
    sources = ensureArray(sources);
    this._popups.forEach((popup) => {
      if (popup.feature?.source && sources.indexOf(popup.feature?.source) >= 0) {
        popup.close();
        this._popups.delete(popup);
      }
    });
  }
}

export { PopupPlugin };
