import { MapPluginBase } from './map-plugin-base.mjs';
import Debug from 'debug';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const debug = Debug("flow:url-sync-plugin");
class UrlSyncPlugin extends MapPluginBase {
  constructor(pluginManager) {
    super(pluginManager);
    __publicField(this, "name", "UrlSyncPlugin");
  }
  initialize() {
    const searchParams = new URLSearchParams(window.location.search);
    const hash = searchParams.get(this.map.id);
    if (hash) {
      const center = this.hashToMapState(hash);
      debug("Setting map position from URL", center);
      this.map.setCenter(center);
    }
  }
  handleMoveEnd(event) {
    const center = this.mapStateToHash(this.map.getCenter());
    const searchParams = new URLSearchParams(window.location.search);
    searchParams.set(this.map.id, center);
    const newUrl = `${location.pathname}?${searchParams.toString()}`;
    debug("Updating URL with map position", center);
    window.history.replaceState({}, "", newUrl);
  }
  mapStateToHash(mapCenter) {
    const state = {
      z: mapCenter.zoom.toFixed(3),
      x: mapCenter.lng.toFixed(3),
      y: mapCenter.lat.toFixed(3),
      pitch: mapCenter.pitch.toFixed(1),
      bearing: mapCenter.bearing.toFixed(1)
    };
    return `${state.z}_${state.x}_${state.y}_${state.pitch}_${state.bearing}`;
  }
  hashToMapState(hash) {
    const [zoom, x, y, pitch, bearing] = hash.split("_");
    return {
      lng: +x,
      lat: +y,
      zoom: +zoom,
      pitch: +pitch,
      bearing: +bearing
    };
  }
}

export { UrlSyncPlugin };
