import Debug from 'debug';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
Debug("flow:image-registry");
const unknownImageSVG = `data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 512 512' class='flowicon'%3E%3C!--! Font Awesome Free 6.1.1 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license/free (Icons: CC BY 4.0, Fonts: SIL OFL 1.1, Code: MIT License) Copyright 2022 Fonticons, Inc.--%3E%3Cpath d='M256 0C114.6 0 0 114.6 0 256s114.6 256 256 256 256-114.6 256-256S397.4 0 256 0zm0 400c-18 0-32-14-32-32s13.1-32 32-32c17.1 0 32 14 32 32s-14.9 32-32 32zm69.1-142L280 286v2c0 13-11 24-24 24s-24-11-24-24v-16c0-8 4-16 12-21l57-34c7-4 11-11 11-19 0-12-10.9-22-22.9-22H238c-12.9 0-22 10-22 22 0 13-11 24-24 24s-24-11-24-24c0-39 31-70 69.1-70h51.1c40.8 0 71.8 31 71.8 70 0 24-13 47-34.9 60z'/%3E%3C/svg%3E`;
class ImageRegistry {
  constructor() {
    __publicField(this, "_registry", /* @__PURE__ */ new Map());
    __publicField(this, "_unknownImage", null);
  }
  get unknownImage() {
    if (!this._unknownImage) {
      const image = new Image(40, 40);
      image.src = unknownImageSVG;
      this._unknownImage = {
        id: "unknown-image",
        src: unknownImageSVG,
        pixelDensity: 2,
        image
      };
    }
    return this._unknownImage;
  }
  /**
   * Register a map icon image.
   *
   * @param id Image ID.
   * @param src Image source URL.
   * @param width Image display width.
   * @param height Image display height.
   * @param pixelDensity Image pixel density. Defaults to `2` - the source image should be @2x resolution to enable better quality rendering on high-resolution displays.
   * @param recolor Set to `true` to allow recoloring of map icon image using map layer style properties.
   */
  registerImage(id, src, width, height, pixelDensity = 2, recolor = false) {
    const entry = { id, src, width, height, pixelDensity, recolor };
    this._registry.set(id, entry);
    return entry;
  }
  /**
   * Get an image registry entry.
   *
   * @param id Image ID.
   * @returns The image registry entry.
   */
  getImage(id) {
    const registryEntry = this._registry.get(id);
    if (!registryEntry) {
      return this.unknownImage;
    }
    if (!registryEntry.image) {
      registryEntry.image = this.loadImage(
        registryEntry.src,
        registryEntry.width,
        registryEntry.height,
        registryEntry.pixelDensity
      );
    }
    return registryEntry;
  }
  /**
   *
   * @param src Image source URL.
   * @param width Image display width in pixels.
   * @param height Image display height in pixels.
   * @param pixelDensity Image pixel density. Defaults to `2` - the source image should be @2x resolution to enable better quality rendering on high-resolution displays.
   */
  loadImage(src, width, height, pixelDensity = 2) {
    const image = new Image();
    image.crossOrigin = "Anonymous";
    image.width = width ? width * pixelDensity : image.width;
    image.height = height ? height * pixelDensity : image.height;
    image.src = src;
    return image;
  }
}

export { ImageRegistry };
