import * as mobx from 'mobx';
import { toJS } from 'mobx';
import { createDecoratorAnnotation, createDecoratorAnnotationWithArg } from '../utils/decorators.mjs';

const SYNC_ANNOTATION_TYPE = "state-sync-annotation";
const REACTIVE_ANNOTATION_TYPE = "state-reactive-annotation";
const serializeRaw = {
  annotationType: SYNC_ANNOTATION_TYPE,
  type: "raw",
  serialize(val) {
    return toJS(val);
  },
  deserialize(val) {
    return val;
  }
};
const serializeString = {
  annotationType: SYNC_ANNOTATION_TYPE,
  type: "string",
  serialize(val) {
    return val.toString();
  },
  deserialize(val) {
    return val;
  }
};
const serializeNumber = {
  annotationType: SYNC_ANNOTATION_TYPE,
  type: "number",
  serialize(val) {
    return val.toString();
  },
  deserialize(val) {
    return +val;
  }
};
const serializeJson = {
  annotationType: SYNC_ANNOTATION_TYPE,
  type: "json",
  serialize(val) {
    return JSON.stringify(val);
  },
  deserialize(val) {
    return JSON.parse(val);
  }
};
const serializeState = {
  annotationType: SYNC_ANNOTATION_TYPE,
  type: "state",
  serialize(val, scope) {
    return val.serialize("raw", scope);
  },
  deserialize(val, target) {
    target.deserialize(val, "raw");
    return target;
  }
};
const serialize = Object.assign(createDecoratorAnnotation(serializeRaw), {
  raw: createDecoratorAnnotation(serializeRaw),
  string: createDecoratorAnnotation(serializeString),
  number: createDecoratorAnnotation(serializeNumber),
  json: createDecoratorAnnotation(serializeJson),
  state: createDecoratorAnnotation(serializeState),
  for: createDecoratorAnnotationWithArg(serializeRaw)
});
function createReactiveAnnotation(mobxAnnotation) {
  return createDecoratorAnnotation({
    annotationType: REACTIVE_ANNOTATION_TYPE,
    mobxAnnotation
  });
}
const observable = Object.assign(createReactiveAnnotation(mobx.observable), {
  deep: createReactiveAnnotation(mobx.observable.deep),
  ref: createReactiveAnnotation(mobx.observable.ref),
  struct: createReactiveAnnotation(mobx.observable.struct),
  shallow: createReactiveAnnotation(mobx.observable.shallow)
});
const computed = Object.assign(createReactiveAnnotation(mobx.computed), {
  struct: createReactiveAnnotation(mobx.computed.struct)
});
const action = createReactiveAnnotation(mobx.action);

export { REACTIVE_ANNOTATION_TYPE, SYNC_ANNOTATION_TYPE, action, computed, observable, serialize };
