import { die } from './assert.mjs';

const storedAnnotationsSymbol = Symbol("flow-rdf-annotations");
function createDecoratorAnnotation(annotation) {
  function decorator(target, property) {
    storeAnnotation(target, property, annotation);
  }
  return Object.assign(decorator, annotation);
}
function createDecoratorAnnotationWithArg(annotation) {
  function decorator(target, property) {
    storeAnnotation(target, property, annotation);
  }
  return (arg) => {
    annotation = Object.assign({}, annotation, { arg });
    return Object.assign(decorator, annotation);
  };
}
function storeAnnotation(prototype, key, annotation) {
  var _a, _b;
  if (!hasProp(prototype, storedAnnotationsSymbol)) {
    addHiddenProp(prototype, storedAnnotationsSymbol, {
      // Inherit annotations
      ...prototype[storedAnnotationsSymbol]
    });
  }
  (_a = prototype[storedAnnotationsSymbol])[_b = annotation.annotationType] ?? (_a[_b] = {});
  assertNotDecorated(prototype, annotation, key);
  prototype[storedAnnotationsSymbol][annotation.annotationType][key] = annotation;
}
function assertNotDecorated(prototype, annotation, key) {
  if (hasProp(prototype[storedAnnotationsSymbol][annotation.annotationType], key)) {
    const fieldName = `${prototype.constructor.name}.prototype.${key.toString()}`;
    const currentAnnotationType = prototype[storedAnnotationsSymbol][key].annotationType_;
    const requestedAnnotationType = annotation.annotationType;
    die(
      `Cannot apply '@${requestedAnnotationType}' to '${fieldName}':
The field is already decorated with '@${currentAnnotationType}'.
Re-decorating fields is not allowed.
Use '@override' decorator for methods overridden by subclass.`
    );
  }
}
function collectStoredAnnotations(target, annotationType) {
  return target[storedAnnotationsSymbol]?.[annotationType] ?? {};
}
function addHiddenProp(object, propName, value) {
  Object.defineProperty(object, propName, {
    enumerable: false,
    writable: true,
    configurable: true,
    value
  });
}
function hasProp(target, prop) {
  return Object.hasOwnProperty.call(target, prop);
}

export { addHiddenProp, collectStoredAnnotations, createDecoratorAnnotation, createDecoratorAnnotationWithArg, hasProp, storeAnnotation, storedAnnotationsSymbol };
