const doNotMemoize = Symbol("Do not memoize");
function memoize(originalFunction, options) {
  const cache = /* @__PURE__ */ new Map();
  const hash = options.hash ? options.hash : (...args) => args[0];
  const context = options.context;
  return function memoizedFunction(...args) {
    const key = hash.apply(context, args);
    if (key === doNotMemoize) {
      return originalFunction.apply(context, args);
    }
    let value;
    if (!cache.has(key)) {
      value = originalFunction.apply(context, args);
      cache.set(key, value);
    } else {
      value = cache.get(key);
    }
    return value;
  };
}

export { doNotMemoize, memoize };
