const isUndefined = (val) => typeof val === "undefined";
const isNull = (val) => val === null;
const isNil = (val) => isUndefined(val) || isNull(val);
const isDefined = (val) => !isNil(val);
const isString = (val) => typeof val === "string";
const hasLength = (val) => isDefined(val) && val.length > 0;
const isStringFull = (val) => isString(val) && hasLength(val);
const isArray = (val) => Array.isArray(val);
const isArrayFull = (val) => Array.isArray(val) && hasLength(val);
const isArrayStrings = (val) => isArrayFull(val) && val.every((v) => isStringFull(v));
const isObject = (val) => typeof val === "object" && !isNull(val);
const isObjectFull = (val) => isObject(val) && hasLength(Object.keys(val));
const isNumber = (val) => typeof val === "number" && !Number.isNaN(val) && Number.isFinite(val);
const isFalse = (val) => val === false;
const isTrue = (val) => val === true;
const isIn = (val, arr) => isArrayFull(arr) && arr.some((o) => val === o);
const isBoolean = (val) => typeof val === "boolean";
const isNumeric = (val) => isString(val) && /^[+-]?([0-9]*[.])?[0-9]+$/.test(val);
const isDate = (val) => val instanceof Date;
const isValue = (val) => isStringFull(val) || isNumber(val) || isBoolean(val) || isDate(val);
const hasValue = (val) => isArrayFull(val) ? val.every((o) => isValue(o)) : isValue(val);
const isFunction = (val) => typeof val === "function";
function isPlainObject(o) {
  if (!isObject(o)) {
    return false;
  }
  const ctor = o.constructor;
  if (ctor === void 0) {
    return true;
  }
  const prot = ctor.prototype;
  if (isObject(prot) === false) {
    return false;
  }
  if (prot.hasOwnProperty("isPrototypeOf") === false) {
    return false;
  }
  return true;
}
const urlRegex = new RegExp(
  "^(https?:\\/\\/)?((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|((\\d{1,3}\\.){3}\\d{1,3}))(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*(\\?[;&a-z\\d%_.~+=-]*)?(\\#[-a-z\\d_]*)?$",
  // fragment locator
  "i"
);
function isURL(val) {
  return isString(val) && !!urlRegex.test(val);
}

export { hasLength, hasValue, isArray, isArrayFull, isArrayStrings, isBoolean, isDate, isDefined, isFalse, isFunction, isIn, isNil, isNull, isNumber, isNumeric, isObject, isObjectFull, isPlainObject, isString, isStringFull, isTrue, isURL, isUndefined, isValue };
